package de.intevation.mxd.reader;

import java.io.IOException;

import java.awt.Color;

import org.apache.log4j.Logger;

import com.esri.arcgis.display.ISymbol;
import com.esri.arcgis.display.HashLineSymbol;
import com.esri.arcgis.display.esriSimpleLineStyle;
import com.esri.arcgis.display.IRgbColor;
import com.esri.arcgis.display.RgbColor;

import com.esri.arcgis.display.esriLineCapStyle;
import com.esri.arcgis.display.esriLineJoinStyle;

import org.w3c.dom.Element;
import de.intevation.mxd.utils.MapToXMLUtils;

/**
 * Reads cartoline symbol information.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class HashLineSymbolReader implements ISymbolReader{

    /**
     * The logger.
     */
    private static final Logger logger =
        Logger.getLogger(HashLineSymbolReader.class);

    /**
     * Private member.
     */
    private Element renderer;
    private HashLineSymbol symbol;
    private MapToXMLUtils util;


    public HashLineSymbolReader(ISymbol symbol)
    throws Exception {
        logger.debug("contructor()");
        if(symbol instanceof HashLineSymbol) {
            this.symbol = (HashLineSymbol)symbol;
        }
        else {
            throw new Exception("Not a HashLineSymbol!");
        }
    }

    /**
     * Setter for the parent XML element.
     *
     * @param parent The XML parent node.
     */
    public void setParent(Element parent) {
        this.renderer = parent;
    }

    /**
     * Setter for XML document helper.
     *
     * @param util The helper class for storing map information.
     */
    public void setUtil(MapToXMLUtils util) {
        this.util = util;
    }

    /**
     * Reads the symbol attributes.
     *
     * @return The XML node.
     */
    public Element read()
    throws IOException {
        logger.debug("read()");
        Element symbolElement;
        try {
            symbolElement = util.addSymbol(renderer);
        }
        catch(Exception e) {
            e.printStackTrace();
            return null;
        }

        symbolElement.setAttribute("name", symbol.getNameString());
        symbolElement.setAttribute("style", "line");
        if(symbol.getColor() instanceof IRgbColor) {
            IRgbColor color = (IRgbColor)symbol.getColor();
            Color c = new Color (
                color.getRed(),
                color.getGreen(),
                color.getBlue());
            symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
        }
        else {
            RgbColor col = new RgbColor();
            col.setRGB(symbol.getColor().getRGB());
            Color c = new Color (
                col.getRed(),
                col.getGreen(),
                col.getBlue());
            symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
        }
        int cap = symbol.getCap();
        String capStyle = "butt";
        switch(cap) {
            case esriLineCapStyle.esriLCSRound: capStyle = "round"; break;
            case esriLineCapStyle.esriLCSSquare: capStyle = "square"; break;
            default: break;
        }
        int join = symbol.getJoin();
        String joinStyle = "bevel";
        switch(join) {
            case esriLineJoinStyle.esriLJSRound: joinStyle = "round"; break;
            case esriLineJoinStyle.esriLJSMitre: joinStyle = "miter"; break;
            default: break;
        }
        symbolElement.setAttribute("cap", capStyle);
        symbolElement.setAttribute("join", joinStyle);
        symbolElement.setAttribute(
            "linestart",
            String.valueOf(symbol.getLineStartOffset()));
        symbolElement.setAttribute(
            "miterlimit",
            String.valueOf(symbol.getMiterLimit()));
        symbolElement.setAttribute(
            "offset",
            String.valueOf(symbol.getOffset()));
        symbolElement.setAttribute("width", String.valueOf(symbol.getWidth()));

        //TODO Read further HashLine specific attributes:
        //     LineSymbol, LineDecoration, Template.

        return symbolElement;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
